﻿local ADDON_NAME, ns = ...
local cfg = ns.cfg

local Ccolor = RAID_CLASS_COLORS[select(2, UnitClass("player"))]

function SetDimensions(frame, width, height, scale, level)
    frame:SetParent(UIParent)
    frame:ClearAllPoints()
    frame:SetFrameStrata("BACKGROUND")
    frame:SetFrameLevel(level)
    frame:SetSize(width, height)
    frame:SetScale(scale)
end

function CreateBackdrop(frame)
    frame:SetBackdrop({bgFile = cfg.flat, edgeFile = cfg.glow, edgeSize = 5, 
        insets = {top = 5, left = 5, bottom = 5, right = 5}})
    frame:SetBackdropColor(0,0,0,0.6)
    frame:SetBackdropBorderColor(0,0,0,1)
end

function colorGradient(perc)
	if perc > 1 then perc = 1 end
	
	local segment, realperc = math.modf(perc*2)
	r1, g1, b1, r2, g2, b2 = unpack(cfg.colorGradient, (segment * 3) + 1)
	return r1 + (r2-r1)*realperc, g1 + (g2-g1)*realperc, b1 + (b2-b1)*realperc
end

function formatMemory(bytes)
		bytes = bytes > 0 and bytes * 1024 or 0.001

		local s = {'B', 'Kb', 'MB', 'GB', 'TB'}
		local e = floor(abs(log(bytes)/log(1024))) or 0
		return format('%.1f '..s[e+1], (bytes/1024^e))
end

SetDimensions(Minimap, 120, 120, 1, 1)
Minimap:EnableMouse(true)
Minimap:SetMaskTexture(cfg.flat)

local bMinimap = CreateFrame("Frame", nil, Minimap)
bMinimap:SetFrameStrata("BACKGROUND")
bMinimap:SetFrameLevel(1)
bMinimap:SetPoint("TOPLEFT", Minimap, "TOPLEFT", -5, 5)
bMinimap:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 5, -5)
CreateBackdrop(bMinimap)

local PMinimap = CreateFrame("Frame", nil, Minimap)
PMinimap:SetFrameStrata("BACKGROUND")
PMinimap:SetFrameLevel(0)
PMinimap:SetPoint("TOPLEFT", Minimap, "TOPLEFT", -100, 100)
PMinimap:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 100, -100)
PMinimap.texture = PMinimap:CreateTexture(nil)
PMinimap.texture:SetAllPoints(PMinimap)
PMinimap.texture:SetTexture("World\\GENERIC\\ACTIVEDOODADS\\INSTANCEPORTAL\\GENERICGLOW2.BLP")
PMinimap.texture:SetVertexColor(Ccolor.r, Ccolor.g, Ccolor.b)
PMinimap.texture:SetBlendMode("ADD")

function fixTooltip(self)
	if self ~= MiniMapMailFrame then
		GameTooltip:SetOwner(self, 'ANCHOR_NONE')
	end
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint('TOPLEFT', Minimap, 'TOPRIGHT', 2, 0)
end
function dropdownOnClick(self)
	GameTooltip:Hide()
	DropDownList1:ClearAllPoints()
	DropDownList1:SetPoint('TOPLEFT', Minimap, 'TOPRIGHT', 2, 0)
end
MiniMapBattlefieldFrame:HookScript('OnClick', dropdownOnClick)

Minimap:SetPoint(unpack(cfg.spawn))
----------------------------------
-- Hide Shit
----------------------------------
for _, hide in next,{MinimapBorder, MinimapBorderTop, MinimapZoomIn, MinimapZoomOut, MiniMapVoiceChatFrame, 
    GameTimeFrame, MinimapZoneTextButton, MiniMapTracking, MiniMapMailBorder, MiniMapBattlefieldBorder, MiniMapWorldMapButton, MiniMapLFGFrameBorder.
    MinimapBackdrop,MinimapCluster,GameTimeFrame} do
    hide:Hide()
end

MinimapNorthTag:SetAlpha(0)

MiniMapMailIcon:SetTexture('Interface\\Minimap\\TRACKING\\Mailbox')
MiniMapMailIcon:SetAllPoints(MiniMapMailFrame)
MiniMapMailBorder:Hide()

MiniMapMailFrame:ClearAllPoints()
MiniMapMailFrame:SetHeight(16)
MiniMapMailFrame:SetWidth(16)
MiniMapMailFrame:SetPoint(unpack(cfg.mailposition))
MiniMapMailFrame:HookScript('OnEnter', fixTooltip)

MiniMapBattlefieldFrame:ClearAllPoints()
MiniMapBattlefieldFrame:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 2, 0)
MiniMapBattlefieldFrame:HookScript('OnEnter', fixTooltip)
MiniMapBattlefieldBorder:Hide()

GuildInstanceDifficulty:ClearAllPoints()
GuildInstanceDifficulty:SetScale(.5)
GuildInstanceDifficulty:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 2, 1)
GuildInstanceDifficulty:SetFrameStrata("HIGH")

MiniMapLFGFrame:ClearAllPoints()
MiniMapLFGFrame:SetParent(Minimap)
MiniMapLFGFrame:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", -2,2)
MiniMapLFGFrameBorder:Hide()
LFGSearchStatus:SetClampedToScreen(true)
LFGDungeonReadyStatus:SetClampedToScreen(true)

Minimap:EnableMouseWheel(true)
Minimap:SetScript('OnMouseWheel', function(self, delta)
    if delta > 0 then
        MinimapZoomIn:Click()
    elseif delta < 0 then
        MinimapZoomOut:Click()
    end
end)

Minimap:SetScript('OnMouseUp', function (self, button)
	if button == 'RightButton' then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, Minimap, (Minimap:GetWidth()+2), (Minimap:GetHeight()))
		GameTooltip:Hide()
	else
		Minimap_OnClick(self)
	end
end)

local function makeit(frame, txt, posx, posy)
    frame:SetHeight(20)
    frame:SetWidth(40)
    frame:SetPoint("LEFT", Minimap, "TOPRIGHT", posx, posy)
    frame:SetParent(Minimap)
    txt:SetPoint("LEFT", frame, "LEFT", 0,0)
    txt:SetFont(cfg.font, 11, "OUTLINE")
end

local id = CreateFrame("Frame", nil, UIParent)
id:SetPoint("TOP", Minimap, "TOP")
id:RegisterEvent("PLAYER_ENTERING_WORLD")
id:RegisterEvent("PLAYER_DIFFICULTY_CHANGED")

local idtext = id:CreateFontString(nil, "OVERLAY")
idtext:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 4, -4)
idtext:SetFont(cfg.font, 14, "THINOUTLINE")
idtext:SetTextColor(Ccolor.r, Ccolor.g, Ccolor.b)

MiniMapInstanceDifficulty:Hide()
function indiff()
	local inInstance, instancetype = IsInInstance()
	local _, _, difficultyIndex, _, _, playerDifficulty, isDynamic = GetInstanceInfo()
	if inInstance and instancetype == "raid" then
		if isDynamic and difficultyIndex == 4 then
			if playerDifficulty == 0 then
				idtext:SetText("25H")
			end
		end
		if isDynamic and difficultyIndex == 3 then
			if playerDifficulty == 0 then
				idtext:SetText("10H")
			end
		end
		if isDynamic and difficultyIndex == 2 then
			if playerDifficulty == 0 then
				idtext:SetText("25")
			end
			if playerDifficulty == 1 then
				idtext:SetText("25H") 
			end
		end
		if isDynamic and difficultyIndex == 1 then
			if playerDifficulty == 0 then
				idtext:SetText("10") 
			end
			if playerDifficulty == 1 then
				idtext:SetText("10H") 
			end
		end
		if not isDynamic then
			if difficultyIndex == 1 then
				idtext:SetText("10") 
			end
			if difficultyIndex == 2 then
				idtext:SetText("25") 
			end
			if difficultyIndex == 3 then
				idtext:SetText("10H") 
			end
			if difficultyIndex == 4 then
				idtext:SetText("25H") 
			end
		end
	end
	if not inInstance then
		idtext:SetText("") 
	end
end
id:SetScript("OnEvent", function() indiff() end)

local Loc = CreateFrame("Frame", "Location", UIParent)
local LocText = Loc:CreateFontString(nil)
makeit(Loc, LocText, 6, -10)

local SubLoc = CreateFrame("Frame", "Sub Location", UIParent)
local SubText = SubLoc:CreateFontString(nil)
SubLoc:SetHeight(20)
SubLoc:SetWidth(40)
SubLoc:SetPoint("CENTER", Minimap, "CENTER", 0, -10)
SubLoc:SetParent(Minimap)
SubText:SetPoint("CENTER", SubLoc, "CENTER", 0,0)
SubText:SetFont(cfg.font, 11, "OUTLINE")
SubText:Hide()

local Lag = CreateFrame("Frame", "Lag", UIParent)
local LagText = Lag:CreateFontString(nil)
makeit(Lag, LagText, 6, -30)

local FPS = CreateFrame("Frame", "FPS", UIParent)
FPS:EnableMouse(true)
local FPSText = FPS:CreateFontString(nil)
makeit(FPS, FPSText, 6, -50)

LoadAddOn('Blizzard_TimeManager')
TimeManagerClockTicker:ClearAllPoints()
TimeManagerClockTicker:SetPoint(unpack(cfg.clockposition))
TimeManagerClockTicker:SetFont(NAMEPLATE_FONT, 14, "OUTLINE")
TimeManagerClockTicker: SetJustifyH('RIGHT')

TimeManagerClockButton:ClearAllPoints()
TimeManagerClockButton:SetPoint("TOPLEFT", TimeManagerClockTicker, -3, 3)
TimeManagerClockButton:SetPoint("BOTTOMRIGHT", TimeManagerClockTicker, 3, -3)
select(1, TimeManagerClockButton:GetRegions()):Hide()
TimeManagerClockButton:HookScript('OnEnter', fixTooltip)
		
local colorlag = function(number)
	if number <= 100 then
		return '|cff00ff00'
	elseif number <= 200 then
		return '|cffffff00'
	else
		return '|cffff0000'
	end
end

local colorfps = function(number)
	if number >= 30 then
		return '|cff00ff00'
	elseif number < 30 then
		return '|cffffff00'
	elseif number < 15 then
		return '|cffff0000'
	end
end

Loc:SetScript('OnMouseUp', function (self)
        if(not CalendarFrame) then LoadAddOn("Blizzard_Calendar") end
        ToggleFrame(WorldMapFrame)
end)

FPS:SetScript('OnMouseUp', function (self)
        if(not CalendarFrame) then LoadAddOn("Blizzard_Calendar") end
        Calendar_Toggle()
end)

local interval = 0
FPS:SetScript('OnUpdate', function(self, elapsed)
	interval = interval - elapsed
    hour, minute = GetGameTime()
	if interval <= 0 then
        FPSText:SetText(string.format('%s%d|r fps', colorfps(GetFramerate()), GetFramerate()))
	    LagText:SetText(string.format('%s%d|r ms', colorlag(select(3,GetNetStats())), select(3,GetNetStats())))
        LocText:SetText(GetZoneText())
        SubText:SetText(GetSubZoneText())
		interval = .5
	end
end)

Minimap:SetScript('OnEnter', function(self)
	if InCombatLockdown() then return end

	UpdateAddOnMemoryUsage()

	local memory = {}
	local total = 0

	for i = 1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local mem = GetAddOnMemoryUsage(i)
			tinsert (memory, {select(2, GetAddOnInfo(i)), mem})
			total = total + mem
		end
	end

	table.sort(memory, function(a, b)
		return a[2] > b[2]
	end)

	fixTooltip(self)
	GameTooltip:AddDoubleLine('Total Usage', formatMemory(total), 1, 1, 1, 1, 1, 1)
	GameTooltip:AddLine(' ')
	for i = 1, #memory do
		GameTooltip:AddDoubleLine(memory[i][1], formatMemory(memory[i][2]), 1, 1, 1, colorGradient(1 - memory[i][2]/total))
	end
	
	GameTooltip:Show()
	SubText:Show()
end)

Minimap:SetScript('OnLeave', function() GameTooltip:Hide() SubText:Hide() end)

local FormatMemoryNumber = function(number)
	if number > 1000 then
		return string.format('%.2f mb', number / 1000)
	else
		return string.format('%.1f kb', number)
	end
end

function GetMinimapShape() return 'SQUARE' end