﻿--Author: Weasoug
local ADDON_NAME, ns = ...
local cfg = ns.cfg

local addonName, eventHandlers = ..., { }

--[[-----------------------------------------------------------------------------
Accept Res
-------------------------------------------------------------------------------]]
if cfg.acceptres then
	eventHandlers['RESURRECT_REQUEST'] = function(name)
		if not (UnitAffectingCombat('player') or UnitAffectingCombat(name)) then
			local delay = GetCorpseRecoveryDelay()
			if delay == 0 then
				AcceptResurrect()
				DoEmote('thank', name)
			else
                                local b = CreateFrame("Button")
				local formattedText = b:GetText(b:SetFormattedText("%d |4second:seconds", delay))
				SendChatMessage("感谢复活我！我复活CD了,还有"..formattedText.."才能起来。", 'WHISPER', nil, name)
			end
		end
	end
end

--[[-----------------------------------------------------------------------------
Battleground Res
-------------------------------------------------------------------------------]]
if cfg.battlegroundres then
	eventHandlers['PLAYER_DEAD'] = function()
			if (MiniMapBattlefieldFrame.status=='active') or (GetRealZoneText()=='Wintergrasp') or (GetRealZoneText()=='TolBarad') then
			RepopMe()
		end
	end
end

--[[-----------------------------------------------------------------------------
Hide Errors
-------------------------------------------------------------------------------]]
if cfg.hideerrors then
	local allowedErrors = { }

	eventHandlers['UI_ERROR_MESSAGE'] = function(message)
		if allowedErrors[message] then
			UIErrorsFrame:AddMessage(message, 1, .1, .1)
		end
	end

	UIErrorsFrame:UnregisterEvent('UI_ERROR_MESSAGE')
end

--[[-----------------------------------------------------------------------------
Ding Announce
-------------------------------------------------------------------------------]]
if cfg.dingannounce then
	eventHandlers['PLAYER_LEVEL_UP'] = function()
		local message = "我升级啦!"
		if GetNumRaidMembers() > 0 then
			SendChatMessage(message, 'RAID')
		elseif GetNumPartyMembers() > 0 then
			SendChatMessage(message, 'PARTY')
		end
		if IsInGuild() then
			SendChatMessage(message, 'GUILD')
		end
	end
end	

--[[-----------------------------------------------------------------------------
Double Click Invite
-------------------------------------------------------------------------------]]
if cfg.doubleclickinvite then
	local function OnDoubleClick(self, button)
		if button == 'LeftButton' then
			if self.buttonType == FRIENDS_BUTTON_TYPE_WOW then
				local name, _, _, _, connected = GetFriendInfo(self.id)
				if connected then
					InviteUnit(name)
				end
			elseif self.buttonType == FRIENDS_BUTTON_TYPE_BNET then
				local _, _, _, toonName, _, client, isOnline = BNGetFriendInfo(self.id)
				if client == 'WoW' and isOnline then
					local _, _, _, realm = BNGetFriendToonInfo(self.id, 1)
					if realm == GetRealmName() then
						InviteUnit(toonName)
					end
				end
			end
		end
	end

	local buttons = FriendsFrameFriendsScrollFrame.buttons
	for index = 1, #buttons do
		buttons[index]:SetScript('OnDoubleClick', OnDoubleClick)
    end
end

--[[-----------------------------------------------------------------------------
Accept Friendly Invites
-------------------------------------------------------------------------------]]
if cfg.acceptfriendlyinvites then
	eventHandlers['PARTY_INVITE_REQUEST'] = function(name)
		local accept
		ShowFriends()
		for index = 1, GetNumFriends() do
			if GetFriendInfo(index) == name then
				accept = true
				break
			end
		end
		if not accept and IsInGuild() then
			GuildRoster()
			for index = 1, GetNumGuildMembers() do
				if GetGuildRosterInfo(index) == name then
					accept = true
					break
				end
			end
		end
		if not accept then
			for index = 1, BNGetNumFriends() do
				local _, _, _, toonName = BNGetFriendInfo(index)
				if toonName == name then
					accept = true
					break
				end
			end
		end
		if accept then
			name = StaticPopup_Visible('PARTY_INVITE')
			if name then
				StaticPopup_OnClick(_G[name], 1)
				return
			end
		else
			SendWho('n-"' .. name .. '"')
		end
	end
end

--[[-----------------------------------------------------------------------------
Autorepair or Autorepairguild / SellJunk
-------------------------------------------------------------------------------]]
if cfg.autorepair or cfg.autorepairguild or cfg.selljunk then
	local handler = ""

	if cfg.selljunk then
		handler = handler .. [[
			local total = 0
			for bag = 0, NUM_BAG_FRAMES do
				for slot = 1, GetContainerNumSlots(bag) do
					local link = GetContainerItemLink(bag, slot)
					if link then
						local _, _, itemRarity, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(link)
						local _, itemCount = GetContainerItemInfo(bag, slot)
						if itemRarity == 0 and itemSellPrice ~= 0 then
							total = total + (itemSellPrice * itemCount)
							UseContainerItem(bag, slot)
						end
					end
				end
			end
			if total > 0 then
				print("出售所有灰色物品获得 " .. MoneyToString(total))
			end
		]]
	end

	if cfg.autorepairguild then
		handler = handler .. [[
			if CanGuildBankRepair() then
				local cost, canRepair = GetRepairAllCost()
				if canRepair and cost > 0 then
					RepairAllItems(1)
					local newCost = GetRepairAllCost()
					if newCost < cost then
						print("使用公会资金修理花费: " .. MoneyToString(cost - newCost))
					end
				end
			end
		]]
	end

	-- Sell first so there is more money for repairs
	if cfg.autorepair then
		handler = handler .. [[
			local cost, canRepair = GetRepairAllCost()
			if canRepair and cost > 0 and cost <= GetMoney() * 0.2 then
				RepairAllItems()
				print("修理花费: " .. MoneyToString(cost))
			end
		]]
	end

	eventHandlers['MERCHANT_SHOW'] = loadstring(([=[
		local cuInd = [[|TInterface\MoneyFrame\UI-CopperIcon:0:1:2:0|t]]
		local agInd = [[|TInterface\MoneyFrame\UI-SilverIcon:0:1:2:0|t ]]
		local auInd = [[|TInterface\MoneyFrame\UI-GoldIcon:0:1:2:0|t ]]

		local function MoneyToString(ammount)
			local cu = ammount %% 100
			ammount = floor(ammount / 100)
			local ag, au = ammount %% 100, floor(ammount / 100)
			if au > 0 then
				return au .. auInd .. ag .. agInd .. cu .. cuInd
			elseif ag > 0 then
				return ag .. agInd .. cu .. cuInd
			end
			return cu .. cuInd
		end

		local function Handler(...)
			%s
		end

		return Handler
	]=]):format(handler))()
end

--[[-----------------------------------------------------------------------------
Easy Add Friend
-------------------------------------------------------------------------------]]
if cfg.easyaddfriend then
local EasyAddFriend = CreateFrame("Frame","EasyAddFriendFrame")
EasyAddFriend:SetScript("OnEvent", function() hooksecurefunc("UnitPopup_ShowMenu", EasyAddFriendCheck) EasyAddFriendSlash() end)
EasyAddFriend:RegisterEvent("PLAYER_LOGIN")

local PopupUnits = {"PARTY", "PLAYER", "RAID_PLAYER", "RAID", "FRIEND", "TEAM", "CHAT_ROSTER", "TARGET", "FOCUS"}

local AddFriendButtonInfo = {
	text = "添加好友",
	value = "ADD_FRIEND",
	func = function() AddFriend(UIDROPDOWNMENU_OPEN_MENU.name) end,
	notCheckable = 1,
}

local CancelButtonInfo = {
	text = "取消",
	value = "CANCEL",
	notCheckable = 1
}

function EasyAddFriendSlash()
	SLASH_EASYADDFRIEND1 = "/add";
	SlashCmdList["EASYADDFRIEND"] = function(msg) if #msg == 0 then DEFAULT_CHAT_FRAME:AddMessage("EasyAddFriend: Use '/add' followed by a character's name to add them to your friends list.") else AddFriend(msg) end end
end
function EasyAddFriendCheck()		
	local PossibleButton = getglobal("DropDownList1Button"..(DropDownList1.numButtons)-1)
	if PossibleButton["value"] ~= "ADD_FRIEND" then
		local GoodUnit = false
		for i=1, #PopupUnits do	
		if OPEN_DROPDOWNMENUS[1]["which"] == PopupUnits[i] then	
			GoodUnit = true
			end
		end
		if UIDROPDOWNMENU_OPEN_MENU["unit"] == "target" and ((not UnitIsPlayer("target")) or (not UnitIsFriend("player", "target"))) then
			GoodUnit = false
		end
		if GoodUnit then				
			local IsAlreadyFriend = false
			for z=1, GetNumFriends() do
				if GetFriendInfo(z) == UIDROPDOWNMENU_OPEN_MENU["name"] or UIDROPDOWNMENU_OPEN_MENU["name"] == UnitName("player") then
					IsAlreadyFriend = true
				end
			end
			if not IsAlreadyFriend then				
				CreateAddFriendButton()
			
			end
		end
	end		
end

function CreateAddFriendButton()

		local CancelButtonFrame = getglobal("DropDownList1Button"..DropDownList1.numButtons)
		CancelButtonFrame:Hide()
		DropDownList1.numButtons = DropDownList1.numButtons - 1
		UIDropDownMenu_AddButton(AddFriendButtonInfo)
		UIDropDownMenu_AddButton(CancelButtonInfo)
	
   end
end

--[[-----------------------------------------------------------------------------
Fatigue Warner
-------------------------------------------------------------------------------]]
if cfg.fatiguewarner then
function FatigueWarner_OnUpdate(self) 
	local timer, value, maxvalue, scale, paused, label = GetMirrorTimerInfo(1) 
	if timer == "EXHAUSTION" then 
-- You can change the sounds by deleting the -- in Front of those PlaySoundFile, so that only one will not have the -- in front of it.
              --PlaySoundFile("Sound\\Creature\\ShadeOfAran\\AranAggro01.wav" , "Master")					
              --PlaySoundFile("Sound\\Creature\\ElderIronbranch\\UR_Ironbranch_Aggro01.wav", "Master")	
                PlaySoundFile("Sound\\Creature\\XT002Deconstructor\\UR_XT002_Special01.wav", "Master")
              --PlaySoundFile("Sound\\Creature\\Hodir\\UR_Hodir_Aggro01.wav", "Master")
	end 
	self:SetScript("OnUpdate", nil) 
end 
 
function FatigueWarner_OnEvent(self) 
	self:SetScript("OnUpdate", FatigueWarner_OnUpdate) 
end 
	  
-- Sinnlos; strip bringt ja irgendwie nichts fiel mir dann auf :>
function FatigueWarner_Strip()
	local FatigueWarner_StripTable = {16, 17, 18, 5, 7, 1, 3, 10, 8, 6, 9}
	local start = 1
	local finish = table.getn(FatigueWarner_StripTable)

	for bag = 0, 4 do
		for slot=1, GetContainerNumSlots(bag) do
			if not GetContainerItemLink(bag, slot) then
				for i = start, finish do
					if GetInventoryItemLink("player", FatigueWarner_StripTable[i]) then
						PickupInventoryItem(FatigueWarner_StripTable[i])
						PickupContainerItem(bag, slot)
						start = i + 1
						break
					end
				end
			end
		end
	end
end

local FatigueWarnerFrame = CreateFrame("frame")
FatigueWarnerFrame:RegisterEvent("MIRROR_TIMER_START")
FatigueWarnerFrame:RegisterEvent("MIRROR_TIMER_STOP")
FatigueWarnerFrame:SetScript("OnEvent", FatigueWarner_OnEvent)
end

--[[-----------------------------------------------------------------------------
Interrupted msg
-------------------------------------------------------------------------------]]
if cfg.interruptedmsg then
eventHandlers['COMBAT_LOG_EVENT_UNFILTERED'] = function(...)
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16 = ...

	if arg2 ~= "SPELL_INTERRUPT" or arg5 ~= UnitName("player") then
		return
	end

        local channel = GetNumRaidMembers() > 0 and "RAID" or GetNumPartyMembers() > 0 and "PARTY"

	if channel then
		SendChatMessage(GetSpellLink(arg12).." 已打断 "..GetSpellLink(arg15), channel)
	end
    end
end
--[[-----------------------------------------------------------------------------
Automaticly accepts/completes quests -- Author: Nightcracker
-------------------------------------------------------------------------------]]
if cfg.autoquests then
local f = CreateFrame("Frame")

local function MostValueable()
	local bestp, besti = 0
	for i=1,GetNumQuestChoices() do
		local link, name, _, qty = GetQuestItemLink("choice", i), GetQuestItemInfo("choice", i)
		local price = link and select(11, GetItemInfo(link))
		if not price then
			return
		elseif (price * (qty or 1)) > bestp then
			bestp, besti = (price * (qty or 1)), i
		end
	end
	if besti then
		local btn = _G["QuestInfoItem"..besti]
		if (btn.type == "choice") then
			btn:GetScript("OnClick")(btn)
		end
	end
end

f:SetScript("OnEvent", function(self, event, ...)
    if (event == "QUEST_DETAIL") then
		AcceptQuest()
		CompleteQuest()
	elseif (event == "QUEST_COMPLETE") then
		if (GetNumQuestChoices() and GetNumQuestChoices() < 1) then
			GetQuestReward()
		else
			MostValueable()
		end
    elseif (event == "QUEST_ACCEPT_CONFIRM") then
		ConfirmAcceptQuest()
	end
end)
f:RegisterEvent("QUEST_ACCEPT_CONFIRM")    
f:RegisterEvent("QUEST_DETAIL")
f:RegisterEvent("QUEST_COMPLETE")

---
QuestInfoDescriptionText.SetAlphaGradient=function() 
return false end

end

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
if next(eventHandlers) then
	local frame = CreateFrame('Frame')
	frame:Hide()

	for event, handler in pairs(eventHandlers) do
		frame[event] = handler
		frame:RegisterEvent(event)
		eventHandlers[event] = nil
	end

	frame:SetScript('OnEvent', function(self, event, ...)
		self[event](...)
	end)
end

--[[-----------------------------------------------------------------------------
World Channel
-------------------------------------------------------------------------------]]
WorldChannel = function()
   local channels = {GetChannelList()}
   local customChannelName = cfg.worldchannelname
   local isInCustomChannel = false
   for i = 1, #channels do
      if channels[i] == customChannelName then
         isInCustomChannel = true
      end
   end
   if isInCustomChannel then
      print("Leave  "..cfg.worldchannelname)
      LeaveChannelByName(customChannelName)
   else
      JoinPermanentChannel(customChannelName,nil,1)
      print("Join  "..cfg.worldchannelname)
      ChatFrame_AddChannel(ChatFrame1,customChannelName)
      ChatFrame_RemoveMessageGroup(ChatFrame1,"CHANNEL")
   end
end

--[[-----------------------------------------------------------------------------
World Channel
-------------------------------------------------------------------------------]]
_G['SLASH_' .. addonName .. 'ReloadUI1'] = strlower("/rl")
SlashCmdList[addonName .. 'ReloadUI'] = ReloadUI